// CS370 - Fall 2016
// Assign03 - Limelight

#ifdef OSX
	#include <GLUT/glut.h>
#else
	#include <GL/glew.h>
	#include <GL/glut.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "materials.h"
#include "lighting.h"
#include "vectorops.h"

// Shader file utility functions
#include "shaderutils.h"

// Shader files
GLchar* lightVertexFile = "lightvert.vs";
GLchar* lightFragmentFile = "lightfrag.fs";
GLchar* defaultVertexFile = "basicvert.vs";
GLchar* defaultFragmentFile = "basicfrag.fs";

// Shader objects
GLuint lightShaderProg;
GLuint defaultShaderProg;
GLuint numLights_param;
GLint numLights = 1;

#define DEG_TO_RAD 0.0175f
#define X 0
#define Y 1
#define Z 2
#define N 8
#define STAGE 1

// Global variables
GLenum lights[4] = { GL_LIGHT0, GL_LIGHT1, GL_LIGHT2, GL_LIGHT3 };

// Light0 (point) Parameters
GLfloat light0_pos[] = { 0.0f, 5.0f, 0.0f, 1.0f };

// Shadow matrices
GLfloat M1_s[16];
GLfloat M2_s[16];
GLfloat M3_s[16];

// Global animation variables
GLint time = 0;
GLint lasttime = 0;
GLint fps = 30;

// Global camera vectors
GLfloat eye[3] = {1.0f,1.0f,1.0f};
GLfloat at[3] = {0.0f,0.0f,0.0f};
GLfloat up[3] = {0.0f,1.0f,0.0f};

// Global screen dimensions
GLfloat ww,hh;

void display();
void render_Scene(bool shadow);
void reshape(int w, int h);
void keyfunc(unsigned char key, int x, int y);
void idlefunc();

int main(int argc, char* argv[])
{
	// Initialize glut
    glutInit(&argc, argv);

	// Initialize window
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitWindowSize(500, 500);
    glutCreateWindow("Limelight");

#ifndef OSX
	// Initialize GLEW - MUST BE DONE AFTER CREATING GLUT WINDOW
	glewInit();
#endif

	// Define callbacks
    glutReshapeFunc(reshape);
    glutDisplayFunc(display);
	glutKeyboardFunc(keyfunc);
	glutIdleFunc(idlefunc);

	// Set background color to grey
	glClearColor(0.6f, 0.6f, 0.6f, 1.0f);

	// Enable depth test
    glEnable(GL_DEPTH_TEST);

	// Load default shader programs
	defaultShaderProg = load_shaders(defaultVertexFile,defaultFragmentFile);
	// Load lighting shader programs
	lightShaderProg = load_shaders(lightVertexFile,lightFragmentFile);
	// Associate num_lights parameter
	numLights_param = glGetUniformLocation(lightShaderProg,"numLights");

	// Start graphics loop
	glutMainLoop();
}

// Display callback
void display()
{
	// Reset background
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	// Set projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	// Adjust viewing volume (orthographic)
	GLfloat xratio = 1.0f;
	GLfloat yratio = 1.0f;
	// If taller than wide adjust y
	if (ww <= hh)
	{
		yratio = (GLfloat)hh / (GLfloat)ww;
	}
	// If wider than tall adjust x
	else if (hh <= ww)
	{
		xratio = (GLfloat)ww / (GLfloat)hh;
	}
	glOrtho(-10.0f*xratio, 10.0f*xratio, -10.0f*yratio, 10.0f*yratio, -10.0f, 10.0f);

	// Set modelview matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(eye[X], eye[Y], eye[Z], at[X], at[Y], at[Z], up[X], up[Y], up[Z]);

	// Render scene without shadows
	render_Scene(false);

	// Flush buffer
	glFlush();

	// Swap buffers
	glutSwapBuffers();
}

void render_Scene(bool shadow)
{
	// Set point light source fixed in world
	set_PointLight(GL_LIGHT0, &white_light, light0_pos);

	// Set number of lights
	glUseProgram(lightShaderProg);
	glUniform1i(numLights_param,numLights);
	// Placeholder stage object
	glPushMatrix();
		set_material(GL_FRONT_AND_BACK, &brass);
		glScalef(16.0, 2.0, 8.0);
		glutSolidCube(1.0);
	glPopMatrix();
}

// Reshape callback
void reshape(int w, int h)
{
	// Set new screen extents
	glViewport(0, 0, w, h);

	// Store new extents
	ww = w;
	hh = h;
}

// Keyboard callback
void keyfunc(unsigned char key, int x, int y)
{
	// Keypress functionality

	// Esc to quit
	if (key==27)
	{
		exit(0);
	}

}

// Idle callback
void idlefunc()
{
	// Time-based Animations

}
